/*
** getmnt v1.0
**
** Show mountpoint
**
** Copyright (C) 14.7.2000 by Andreas Ley <andreas.ley@kit.edu>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** This program has been tested on a HP9000/715 with HP-UX A.09.07
** In this environment, neither lint -u nor gcc -Wall -pedantic -fsyntax-only
** produce any messages. If you encounter any errors or need to make any
** changes to port it to another platform, please contact me.
**
** Version history
**
** Version 1.0 - 14.7.2000
**	Initial version
*/

static char copyright[] = "@(#)Copyright (C) 2000 by Andreas Ley (andreas.ley@kit.edu)";
static char sccsid[] = "@(#)getmnt v1.0 - Show mountpoint";


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef __hpux
#include <time.h>
#endif
#ifdef _AIX
#include <fshelp.h>
#include <sys/vfs.h>
#include <sys/mntctl.h>
#include <sys/vmount.h>
#else
#include <mntent.h>
#endif

#ifdef MNT_MNTTAB
#define MOUNTED	MNT_MNTTAB
#endif

#ifndef FALSE
#define	FALSE	(0)		/* This is the naked Truth */
#define	TRUE	(1)		/* and this is the Light */
#endif


char	*image;
int	debug=0;
#ifdef _AIX
enum	{ALL, NAME, PATH, OPTIONS, ID, NUMBER, TIME} mode=ALL;
#else
#ifdef __hpux
enum	{ALL, NAME, PATH, TYPE, OPTIONS, FREQUENCY, PASS, TIME} mode=ALL;
#else
enum	{ALL, NAME, PATH, TYPE, OPTIONS, FREQUENCY, PASS} mode=ALL;
#endif
#endif


static void showmount(fname)
char	*fname;
{
	struct stat	buf,mntbuf;
#ifdef _AIX
	int		size;
	char		*buffer,*mnt;
#else
	FILE		*mnttab;
	struct mntent	*mnt;
#endif

	if (debug)
		(void)fprintf(stderr,"[%d] showmount(\"%s\")\n",__LINE__,fname);

	if (fname) {
		if ((stat(fname,&buf))) {
			(void)fprintf(stderr,"%s: cannot stat ",image);
			perror(fname);
			return;
		}
		if (debug)
			(void)fprintf(stderr,"[%d] dev=0x%08lx\n",__LINE__,buf.st_dev);
	}

#ifdef _AIX
	mntctl(MCTL_QUERY,sizeof(size),(struct vmount *)&size);
	if (!(buffer=malloc(size))) {
		(void)fprintf(stderr,"%s: cannot malloc %d bytes",image,size);
		perror(NULL);
		return;
	}
	mntctl(MCTL_QUERY,size,(struct vmount *)buffer);
	for (mnt=buffer;mnt<buffer+size;mnt+=((struct vmount *)mnt)->vmt_length) {
		if (debug)
			(void)fprintf(stderr,"[%d] mnt=0x%08lx buffer=0x%08lx buffer+size=0x%08lx mnt->vmt_length==0x%08lx\n",__LINE__,mnt,buffer,buffer+size,((struct vmount *)mnt)->vmt_length);
		if (fname) {
			if ((stat(vmt2dataptr(((struct vmount *)mnt),VMT_STUB),&mntbuf))) {
				(void)fprintf(stderr,"%s: cannot stat ",image);
				perror(vmt2dataptr(((struct vmount *)mnt),VMT_STUB));
				return;
			}
			if (debug)
				(void)fprintf(stderr,"[%d] mntdev=0x%08lx\n",__LINE__,mntbuf.st_dev);
		}
		if (!fname||buf.st_dev==mntbuf.st_dev) {
			switch (mode) {
			case NAME:
				if (((struct vmount *)mnt)->vmt_flags&MNT_REMOTE&&vmt2datasize(((struct vmount *)mnt),VMT_HOST))
					(void)printf("%s:%s\n",vmt2dataptr(((struct vmount *)mnt),VMT_HOSTNAME),vmt2dataptr(((struct vmount *)mnt),VMT_OBJECT));
				else
					(void)printf("%s\n",vmt2dataptr(((struct vmount *)mnt),VMT_OBJECT));
				break;
			case PATH:
				(void)printf("%s\n",vmt2dataptr(((struct vmount *)mnt),VMT_STUB));
				break;
			case OPTIONS:
				(void)printf("%s\n",vmt2dataptr(((struct vmount *)mnt),VMT_ARGS));
				break;
			case ID:
				(void)printf("%08lx\n",((struct vmount *)mnt)->vmt_fsid);
				break;
			case NUMBER:
				(void)printf("%08lx\n",((struct vmount *)mnt)->vmt_vfsnumber);
				break;
			case TIME:
				(void)printf("%s",ctime(&((struct vmount *)mnt)->vmt_time));
				break;
			default:
				if (((struct vmount *)mnt)->vmt_flags&MNT_REMOTE&&vmt2datasize(((struct vmount *)mnt),VMT_HOST))
					(void)printf("Name:\t\t%s:%s\n",vmt2dataptr(((struct vmount *)mnt),VMT_HOSTNAME),vmt2dataptr(((struct vmount *)mnt),VMT_OBJECT));
				else
					(void)printf("Name:\t\t%s\n",vmt2dataptr(((struct vmount *)mnt),VMT_OBJECT));
				(void)printf("Mount point:\t%s\n",vmt2dataptr(((struct vmount *)mnt),VMT_STUB));
				(void)printf("Options:\t%s\n",vmt2dataptr(((struct vmount *)mnt),VMT_ARGS));
				(void)printf("FS ID:\t\t0x%08lx\n",((struct vmount *)mnt)->vmt_fsid);
				(void)printf("Mount ID:\t0x%08lx\n",((struct vmount *)mnt)->vmt_vfsnumber);
				(void)printf("Mount time:\t%s\n",ctime(&((struct vmount *)mnt)->vmt_time));
			}
		}
	}
#else
	if (!(mnttab=setmntent(MOUNTED,"r"))) {
		(void)fprintf(stderr,"%s: cannot open ",image);
		perror(MOUNTED);
		return;
	}

	while ((mnt=getmntent(mnttab))) {
		if (fname) {
			if ((stat(mnt->mnt_dir,&mntbuf))) {
				(void)fprintf(stderr,"%s: cannot stat ",image);
				perror(mnt->mnt_dir);
				return;
			}
			if (debug)
				(void)fprintf(stderr,"[%d] mntdev=0x%08lx\n",__LINE__,mntbuf.st_dev);
		}
		if (!fname||buf.st_dev==mntbuf.st_dev) {
			switch (mode) {
			case NAME:
				(void)printf("%s\n",mnt->mnt_fsname);
				break;
			case PATH:
				(void)printf("%s\n",mnt->mnt_dir);
				break;
			case TYPE:
				(void)printf("%s\n",mnt->mnt_type);
				break;
			case OPTIONS:
				(void)printf("%s\n",mnt->mnt_opts);
				break;
			case FREQUENCY:
				(void)printf("%d\n",mnt->mnt_freq);
				break;
			case PASS:
				(void)printf("%d\n",mnt->mnt_passno);
				break;
#ifdef __hpux
			case TIME:
				(void)printf("%s",ctime(&mnt->mnt_time));
				break;
#endif
			default:
				(void)printf("Name:\t\t%s\n",mnt->mnt_fsname);
				(void)printf("Mount point:\t%s\n",mnt->mnt_dir);
				(void)printf("Type:\t\t%s\n",mnt->mnt_type);
				(void)printf("Options:\t%s\n",mnt->mnt_opts);
				(void)printf("Dump frequency:\t%d\n",mnt->mnt_freq);
#ifdef __hpux
				(void)printf("Pass number:\t%d\n",mnt->mnt_passno);
				(void)printf("Mount time:\t%s\n",ctime(&mnt->mnt_time));
#else
				(void)printf("Pass number:\t%d\n\n",mnt->mnt_passno);
#endif
			}
		}
	}
#endif
}



static void usage()
{
#ifdef _AIX
	(void)fprintf(stderr,"Usage: %s [-n|-m|-o|-i|-I|-T] [filename...]\n",image);
#else
#ifdef __hpux
	(void)fprintf(stderr,"Usage: %s [-n|-m|-t|-o|-f|-p|-T] [filename...]\n",image);
#else
	(void)fprintf(stderr,"Usage: %s [-n|-m|-t|-o|-f|-p] [filename...]\n",image);
#endif
#endif
	(void)fprintf(stderr,"-n  Show file system name\n");
	(void)fprintf(stderr,"-m  Show file system mount point\n");
#ifndef _AIX
	(void)fprintf(stderr,"-t  Show file system type\n");
#endif
	(void)fprintf(stderr,"-o  Show file system options\n");
#ifdef _AIX
	(void)fprintf(stderr,"-i  Show file system id\n");
	(void)fprintf(stderr,"-I  Show file system mount id\n");
#else
	(void)fprintf(stderr,"-f  Show file system dump frequency\n");
	(void)fprintf(stderr,"-p  Show file system pass number\n");
#endif
#if defined(_AIX) && defined(__hpux)
	(void)fprintf(stderr,"-T  Show file system mount time\n");
#endif
	exit(1);
}


int main(argc,argv)
int	argc;
char	*argv[];
{
	int		c;
	extern char	*optarg;
	extern int	optind;

	if ((image=strrchr(argv[0],'/')))
		image++;
	else
		image=argv[0];

#ifdef _AIX
	while ((c=getopt(argc,argv,"DnmoiITvh?"))!=EOF)
#else
#ifdef __hpux
	while ((c=getopt(argc,argv,"DnmtofpTvh?"))!=EOF)
#else
	while ((c=getopt(argc,argv,"Dnmtofpvh?"))!=EOF)
#endif
#endif
		switch ((char)c) {
		case 'D':
			debug++;
			break;
		case 'n':
			if (mode!=ALL)
				usage();
			mode=NAME;
			break;
		case 'm':
			if (mode!=ALL)
				usage();
			mode=PATH;
			break;
#ifndef _AIX
		case 't':
			if (mode!=ALL)
				usage();
			mode=TYPE;
			break;
#endif
		case 'o':
			if (mode!=ALL)
				usage();
			mode=OPTIONS;
			break;
#ifdef _AIX
		case 'i':
			if (mode!=ALL)
				usage();
			mode=ID;
			break;
		case 'I':
			if (mode!=ALL)
				usage();
			mode=NUMBER;
			break;
#else
		case 'f':
			if (mode!=ALL)
				usage();
			mode=FREQUENCY;
			break;
		case 'p':
			if (mode!=ALL)
				usage();
			mode=PASS;
			break;
#endif
#if defined(_AIX) && defined(__hpux)
		case 't':
			if (mode!=ALL)
				usage();
			mode=TIME;
			break;
#endif
		case 'v':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
			exit(0);
			break;
		case 'h':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
		/*FALLTHROUGH*/
		case '?':
			usage();
		}

	if (optind<argc)
		for (;optind<argc;optind++)
			showmount(argv[optind]);
	else
		showmount(NULL);

	return(0);
}
